;**********************************************
; mSZ -define a string
; Str_Copy
; Str_Copy_Nr
; Str_Len
; StrConCat3
; Find_Str_In_Str
; Get_Word
; Str_Compare
;*****************


mSZ MACRO a_str:VARARG
	LOCAL	l_sz
.DATA
	l_sz	db a_str,0
ALIGN 4
.CODE
	EXITM <offset l_sz>
ENDM


;----------------------
; OUT :ZERO if EQUAL
;----------------------
Str_Compare PROC USES EDI ESI a_lpsz1,a_lpsz2:DWORD

	mov	esi,[a_lpsz1]
	mov	edi,[a_lpsz2]
@@loop_cmp:
	mov	al,[esi]
	mov	dl,[edi]
	
	cmp	al,dl
	jnz	@@not_equal
	
	test	al,al
	jz	@@equal
	
	inc	esi
	inc	edi
	jmp	@@loop_cmp
	
@@equal:
	xor	eax,eax
ret
@@not_equal:
	or	eax,1
ret
Str_Compare ENDP

;-----------------------------------------
; test if a char is a word separator
; IN: al=char
; USES: edi,ecx,edx
; OUT: ZERO if separator
;-----------------------------------------
mChar_Separator? MACRO
LOCAL	l_loop,l_separator,l_end

	mov	edi,[a_lpsz_separators]
	mov	ecx,[a_separators_count]
l_loop:
	mov	dl,[edi]
	cmp	al,dl
	jz	l_separator
	
	inc	edi
	dec	ecx
	jnz	l_loop
	
;here is not separator
	or	edx,1
	jmp	l_end
	
l_separator:
	xor	edx,edx
l_end:
ENDM

;----------------------------------------
;
; returns lp to word in string
; writes 0 at the end of word!
; OUT: flag ZERO=ok , !ZERO =err
;--------------------------------------
Get_Word PROC USES ESI ECX EDI EBX a_lpsz,a_lpsz_separators,a_separators_count:DWORD
	LOCAL	l_lpsz_word,l_word_size:DWORD
	
;search for first non-separator char
	mov	esi,[a_lpsz]
@@loop_first_char:
	mov	al,[esi]
	
	test	al,al
	jz	@@err
	
	mChar_Separator?
	jnz	@@end_first_char
	
	inc	esi
	jmp	@@loop_first_char
	
@@end_first_char:
	
;esi = lp to start of word
	mov	ebx,esi
	
@@loop_next_separator:
	mov	al,[esi]
	inc	esi
	
;	test	al,al
;	jz	@@err
	
	mChar_Separator?
;	jnz	@@loop_next_separator
	jz	@@end_word
	
	test	al,al
	jz	@@end_word		;overprotection for end of string
	
	jmp	@@loop_next_separator
	
@@end_word:
	mov	BYTE PTR [esi-1],0	;mark end of word
	
	mov	eax,ebx
	xor	edx,edx			;flag zero
ret

@@err:
	or	edx,1
ret
Get_Word ENDP

mNext_Word MACRO a_lp
	invoke	Str_Len,a_lp
	inc	eax
	add	a_lp,eax
ENDM

;--------------------------------------
; returns a pointer to NEXT char after
; substring / 0 if error
;--------------------------------------
Find_Str_In_Str PROC a_lp_str,a_lp_substr:DWORD
	LOCAL	l_src:DWORD

	mov	eax,[a_lp_str]
	mov	[l_src],eax

@@loop_search:
	mov	esi,[l_src]
	mov	edi,[a_lp_substr]
@@loop_test:
	mov	bl,[edi]
	test	bl,bl
	jz	@@found		;found keyword

	mov	al,[esi]
	test	al,al
	jz	@@no_found		;if we reached end in string - no find

	cmp	al,bl
	jnz	@@inc_char

	inc	esi
	inc	edi
	jmp	@@loop_test

;----------------------
; inc crt pos in string
;----------------------
@@inc_char:
	inc	[l_src]
	jmp	@@loop_search

@@no_found:
	xor	eax,eax
ret
@@found:
;mov	eax,[l_src]
;sub	eax,[a_lpsz]
	mov	eax,esi
ret

Find_Str_In_Str ENDP


;==========================================================
;
; Str_Copy
;
;IN:
;	lpsz_src,lpsz_dest
;OUT:
;	eax==Length copied (without 0)
;==========================================================

Str_Copy PROC USES esi edi a_lpsz_src,a_lpsz_dest:DWORD

	mov	esi,[a_lpsz_src]
	mov	edi,[a_lpsz_dest]
@@copy_sz:
	lodsb
	stosb
	test	al,al
	jnz	@@copy_sz

;return length copied
	mov	eax,esi
	sub	eax,[a_lpsz_src]
	dec	eax
ret
Str_Copy ENDP


;==========================================================
;
; copies a max number of chars from a string to another
;
;IN:
;	lpsz_src,lpsz_dest,nr
;OUT:
;	eax==Length copied (without 0)
; Resulting string is ZERO terminated
;==========================================================

Str_Copy_Nr PROC USES ESI EDI a_lpsz_src,a_lpsz_dest,a_count:DWORD

	mov	esi,[a_lpsz_src]
	mov	edi,[a_lpsz_dest]
	mov	edx,[a_count]
	test	edx,edx
	jz	@@end
@@copy_sz:
	lodsb
	stosb
	
	test	al,al
	jz	@@end
	
	dec	edx
	jnz	@@copy_sz
	
	mov	BYTE PTR [edi],0
	
@@end:
;return length copied
	mov	eax,[a_count]
	sub	eax,edx
ret
Str_Copy_Nr ENDP


;-----------------------------------------------
;
;input:		-offset of string
;output:	-eax=length - zero
;
;-----------------------------------------------
Str_Len PROC USES ESI ECX a_lpsz_string:DWORD

	mov	esi,[a_lpsz_string]
	mov	ecx,-1			; ecx counter
	cld

@@len_loop:
	lodsb
	inc	ecx
	test	al,al
	jnz	@@len_loop

	mov	eax,ecx
ret
Str_Len ENDP


;-----------------------------------------------
;
;input:		-offset of strings
;output:	-
;
;-----------------------------------------------
Str_ConCat3 PROC USES esi edi a_lpsz_src1,a_lpsz_src2,a_lpsz_src3,a_lpsz_dest1:DWORD

	mov	esi,[a_lpsz_src1]
	mov	edi,[a_lpsz_dest1]

@@copy1:
	lodsb
	stosb
	test	al,al
	jnz	@@copy1

	mov	esi,[a_lpsz_src2]
	dec	edi		;overwrite zero
@@copy2:
	lodsb
	stosb
	test	al,al
	jnz	@@copy2

	mov	esi,[a_lpsz_src3]
	dec	edi		;overwrite zero
@@copy3:
	lodsb
	stosb
	test	al,al
	jnz	@@copy3
ret
Str_ConCat3 ENDP

